function phaselist=mkphasecatalog(phasetype);
% mkphasecatalog.........return list of computable phases
%
% call:  phaselist=mkphasecatalog(phasetype);
%
%        phasetype: 'P': get a list of simple P-phases ('P','PKP' and the like)
%                   'Pmult': multiples of P phases ('PP','PKPPKP' etc)
%                   'S': get a list of simple S-phases
%                   'Smult': multiples of S phases
%                   'mix': get a list of mixed phases ('PS' and the like)
%                   'mixmult': multiples of mixed phases
%                   'allsimple': all simple P and S phases
%                   'all': get a list of all phases
%                   PHASETYPE is not case sensitive.
%
% result: phaselist: stringmatrix generated by STRVCAT, containing the phase names
%
% The discrimination of P- and S-phases by their first leg is somewhat arbitrary,
% but necessary since some programs try to distinguish (like MKANALYZEMODEL).
%
% The main purpose of this function is to have a central list of phases available
% in MKX4P and MKTIM4P. So if new phases are implemented, only mkphasecatalog has to
% be updated and all functions that produce comprehensive catalogues of times or curves
% or whatever are up-to-date automaticaly.
%
% In a later version of the travel time package, a phase name parser may make this
% obsolete. MKPHASECATALOG will then return a list of important phases only, not a
% comprehensive list of all. But that is future work.
%
% TTBOX handles depth phases recursively by combining an upgoing leg with a standard
% phase for depth zero. So depth phases are not contained in this catalogue.
%
% Martin Knapmeyer, 04.06.2002

%%% interpret input (or, to be precise, absence of input)
if nargin==0
   phasetype='all';
end;

%%% make input case independent
phasetype=lower(phasetype);


%%% prepare result
phaselist=[];

%%% the "data base"
pphases=strvcat('P','PKP','PKIKP','PcP','PKiKP','PKKP');
pmult=strvcat('PP','PPP','PcPPcP','PKPPKP','PKIKPPKIKP');
sphases=strvcat('S','SKS','SKIKS','ScS','SKKS');
smult=strvcat('SS','SSS','ScSScS','SKSSKS');
mixphases=strvcat('SP','PS','ScP','PcS');
mixmult=strvcat('ScPScP','PcSPcS');

switch phasetype
   case {'p'}
      phaselist=pphases;
   case {'pmult'}
      phaselist=pmult;
   case {'s'}
      phaselist=sphases;
   case {'smult'}
      phaselist=smult;
   case {'mix'}
      phaselist=mixphases;
   case {'mixmult'}
      phaselist=mixmult;
   case {'allsimple'}
      phaselist=strvcat(pphases,mixphases,sphases);
   case {'all'}
      phaselist=strvcat(pphases,pmult,mixphases,mixmult,sphases,smult);
   otherwise
       error(['MKPHASECATALOG: unrecognized phase list type ' phasetype]);
end; % switch


      
